export load

import JSON3
import YAML as YAMLlib

"""
    load(files::T) where T <: AbstractFile

Generic loader of different file/s formats.
"""
function load(::T) where {T<:AbstractFile} end
function load(::Type{T}, files::String...) where {T<:AbstractFile} end


"""
    load(file::JSON{1})

Open a JSON file and parse its data in a dictionary.
"""
function load(file::JSON{1})
    open(filepath(file), "r") do f
        data = JSON3.read(f)
        return Dict(data)
    end
end

"""
    load(file::TXT{1})

Open a TEXT file and parse its data in a list of strings.
"""
function load(file::TXT{1})
    readlines(filepath(file))
end

"""
    load(file::YAML{1})

Open a YAML file and parse its data in a dictionary.
"""
function load(file::YAML{1})
    YAMLLib.load_file(filepath(file); dicttype = Dict{Symbol,Any})
end
